import { Maybe } from './types';
/**
 * Operators used by Sequence
 */
export declare function filter<T>(fnFilter: (t: T) => boolean, i: Iterable<T>): Generator<T, void, unknown>;
export declare function skip<T>(n: number, i: Iterable<T>): IterableIterator<T>;
export declare function take<T>(n: number, i: Iterable<T>): IterableIterator<T>;
/**
 * Concat two iterables together
 */
export declare function concat<T>(i: Iterable<T> | IterableIterator<T>, j: Iterable<T> | IterableIterator<T>): IterableIterator<T>;
export declare function concatMap<T, U>(fn: (t: T) => Iterable<U>, i: Iterable<T> | IterableIterator<T>): IterableIterator<U>;
/**
 * Combine two iterables together using fnMap function.
 */
export declare function combine<T, U, V>(fnMap: (t: T, u?: U) => V, i: Iterable<T> | IterableIterator<T>, j: Iterable<U> | IterableIterator<U>): IterableIterator<V>;
/**
 * apply a mapping function to an Iterable.
 */
export declare function map<T, U>(fnMap: (t: T) => U): (i: Iterable<T>) => IterableIterator<U>;
export declare function map<T, U>(fnMap: (t: T) => U, i: Iterable<T>): IterableIterator<U>;
export declare function scan<T>(i: Iterable<T>, fnReduce: (prevValue: T, curValue: T, curIndex: number) => T): IterableIterator<T>;
export declare function scan<T>(i: Iterable<T>, fnReduce: (prevValue: T, curValue: T, curIndex: number) => T, initValue: T): IterableIterator<T>;
export declare function scan<T, U>(i: Iterable<T>, fnReduce: (prevValue: U, curValue: T, curIndex: number) => U, initValue: U): IterableIterator<U>;
export declare function all<T>(fn: (t: T) => boolean, i: Iterable<T>): boolean;
export declare function any<T>(fn: (t: T) => boolean, i: Iterable<T>): boolean;
export declare function count<T>(i: Iterable<T>): number;
export declare function first<T>(fn: (t: T) => boolean, defaultValue: T, i: Iterable<T>): T;
export declare function forEach<T>(fn: (t: T, index: number) => void, i: Iterable<T>): void;
export declare function max<T, U>(selector: (t: T) => U, i: Iterable<T>): Maybe<T>;
export declare function min<T, U>(selector: (t: T) => U, i: Iterable<T>): Maybe<T>;
export declare function reduce<T, U>(fnReduce: (prevValue: U, curValue: T, curIndex: number) => U, initialValue: U, i: Iterable<T>): U;
export declare function reduce<T>(fnReduce: (prevValue: T, curValue: T, curIndex: number) => T, initialValue: T, i: Iterable<T>): T;
export declare function reduce<T>(fnReduce: (prevValue: T, curValue: T, curIndex: number) => T, initialValue: Maybe<T>, i: Iterable<T>): Maybe<T>;
/**
 * Convert an Iterator into an IterableIterator
 */
export declare function makeIterable<T>(i: Iterator<T> | IterableIterator<T>): IterableIterator<T>;
export declare function isIterable<T>(i: Iterator<T> | IterableIterator<T>): i is IterableIterator<T>;
/**
 * Creates a scan function that can be used in a map function.
 */
export declare function scanMap<T>(accFn: (acc: T, value: T) => T, init?: T): ((value: T) => T);
export declare function scanMap<T, U>(accFn: (acc: U, value: T) => U, init: U): ((value: T) => U);
